@@ -17,6 +17,7 @@
 
 #include "emu.h"
 
+#include "adbmodem.h"
 #include "egret.h"
 #include "macadb.h"
 #include "macrtc.h"
@@ -24,6 +25,7 @@
 #include "mactoolbox.h"
 #include "rbv.h"
 
+#include "bus/nscsi/cd.h"
 #include "bus/nscsi/devices.h"
 #include "bus/nubus/nubus.h"
 #include "bus/nubus/cards.h"
@@ -44,8 +46,8 @@
 
 namespace {
 
-static constexpr u32 C7M = 7833600;
-static constexpr u32 C15M = (C7M * 2);
+static constexpr XTAL C15M = 31.3344_MHz_XTAL / 2;
+static constexpr XTAL C7M = 31.3344_MHz_XTAL / 4;
 
 class maciici_state : public driver_device
 {
@@ -57,6 +59,7 @@ public:
 		m_rbv(*this, "rbv"),
 		m_macadb(*this, "macadb"),
 		m_ram(*this, RAM_TAG),
+		m_adbmodem(*this, "adbmodem"),
 		m_asc(*this, "asc"),
 		m_scsibus1(*this, "scsi"),
 		m_ncr5380(*this, "scsi:7:ncr5380"),
@@ -65,16 +68,18 @@ public:
 		m_floppy(*this, "fdc:%d", 0U),
 		m_scc(*this, "scc"),
 		m_rtc(*this, "rtc"),
-		m_egret(*this, "egret")
+		m_egret(*this, "egret"),
+		m_config(*this, "config")
 	{
 	}
 
+	void maciixi_base(machine_config &config);
 	void maciici(machine_config &config);
 	void maciisi(machine_config &config);
 
 protected:
-	virtual void machine_start() override;
-	virtual void machine_reset() override;
+	virtual void machine_start() override ATTR_COLD;
+	virtual void machine_reset() override ATTR_COLD;
 
 private:
 	required_device<m68030_device> m_maincpu;
@@ -82,6 +87,7 @@ private:
 	required_device<rbv_device> m_rbv;
 	required_device<macadb_device> m_macadb;
 	required_device<ram_device> m_ram;
+	optional_device<adbmodem_device> m_adbmodem;
 	required_device<asc_device> m_asc;
 	required_device<nscsi_bus_device> m_scsibus1;
 	required_device<ncr5380_device> m_ncr5380;
@@ -91,6 +97,7 @@ private:
 	required_device<z80scc_device> m_scc;
 	optional_device<rtc3430042_device> m_rtc;
 	optional_device<egret_device> m_egret;
+	optional_ioport m_config;
 
 	void set_via2_interrupt(int value);
 	void field_interrupts();
@@ -111,15 +118,15 @@ private:
 	void via_out_b(uint8_t data);
 	void via_out_b_iisi(uint8_t data);
 	void via_sync();
-	DECLARE_WRITE_LINE_MEMBER(via_irq);
-	WRITE_LINE_MEMBER(via_out_cb2);
-	WRITE_LINE_MEMBER(via_out_cb2_iisi);
-	WRITE_LINE_MEMBER(adb_irq_w) { m_adb_irq_pending = state; }
-	DECLARE_WRITE_LINE_MEMBER(scc_irq_w);
+	void via_irq(int state);
+	void via_out_cb2(int state);
+	void via_out_cb2_iisi(int state);
+	void adb_irq_w(int state) { m_adb_irq_pending = state; }
+	void scc_irq_w(int state);
 
 	uint32_t rom_switch_r(offs_t offset);
 
-	void maciici_map(address_map &map);
+	void maciici_map(address_map &map) ATTR_COLD;
 
 	u16 scc_r(offs_t offset)
 	{
@@ -141,7 +148,7 @@ private:
 		m_maincpu->pulse_input_line(M68K_LINE_BUSERROR, attotime::zero);
 	}
 
-	WRITE_LINE_MEMBER(egret_reset_w)
+	void egret_reset_w(int state)
 	{
 		m_maincpu->set_input_line(INPUT_LINE_HALT, state);
 		m_maincpu->set_input_line(INPUT_LINE_RESET, state);
@@ -169,14 +176,9 @@ private:
 			m_fdc->write((offset >> 8) & 0xf, data & 0xff);
 		else
 			m_fdc->write((offset >> 8) & 0xf, data >> 8);
-	}
 
-	WRITE_LINE_MEMBER(write_6015)
-	{
-		if (state)
-		{
-			m_macadb->adb_vblank();
-		}
+		if (!machine().side_effects_disabled())
+			m_maincpu->adjust_icount(-5);
 	}
 };
 
@@ -198,6 +200,11 @@ void maciici_state::machine_reset()
 		m_maincpu->set_input_line(INPUT_LINE_HALT, ASSERT_LINE);
 	}
 
+	if (m_config)
+	{
+		m_maincpu->set_fpu_enable(BIT(m_config->read(), 0));
+	}
+
 	// put ROM mirror at 0
 	address_space &space = m_maincpu->space(AS_PROGRAM);
 	const u32 memory_size = std::min((u32)0x3fffff, m_rom_size);
@@ -258,13 +265,13 @@ void maciici_state::field_interrupts()
 	}
 }
 
-WRITE_LINE_MEMBER(maciici_state::via_irq)
+void maciici_state::via_irq(int state)
 {
 	m_via_interrupt = state;
 	field_interrupts();
 }
 
-WRITE_LINE_MEMBER(maciici_state::scc_irq_w)
+void maciici_state::scc_irq_w(int state)
 {
 	m_scc_interrupt = state;
 	field_interrupts();
@@ -339,15 +346,13 @@ uint8_t maciici_state::via_in_a_iisi()
 
 uint8_t maciici_state::via_in_b()
 {
-	u8 val = m_macadb->get_adb_state() << 4;
+	u8 val = m_rtc->data_r();
 
 	if (!m_adb_irq_pending)
 	{
 		val |= 0x08;
 	}
 
-	val |= m_rtc->data_r();
-
 	return val;
 }
 
@@ -372,9 +377,9 @@ void maciici_state::via_out_a(uint8_t data)
 void maciici_state::via_out_b(uint8_t data)
 {
 	//  printf("%s VIA1 OUT B: %02x\n", machine().describe_context().c_str(), data);
-	m_macadb->mac_adb_newaction((data & 0x30) >> 4);
+	m_adbmodem->set_via_state((data & 0x30) >> 4);
 
-	m_rtc->ce_w(BIT(data, 2));
+m_rtc->ce_w(BIT(data, 2));
 	m_rtc->data_w(BIT(data, 0));
 	m_rtc->clk_w(BIT(data, 1));
 }
@@ -385,12 +390,12 @@ void maciici_state::via_out_b_iisi(uint8_t data)
 	m_egret->set_sys_session(BIT(data, 5));
 }
 
-WRITE_LINE_MEMBER(maciici_state::via_out_cb2)
+void maciici_state::via_out_cb2(int state)
 {
-	m_macadb->adb_data_w(state);
+//  m_macadb->adb_data_w(state);
 }
 
-WRITE_LINE_MEMBER(maciici_state::via_out_cb2_iisi)
+void maciici_state::via_out_cb2_iisi(int state)
 {
 	m_egret->set_via_data(state & 1);
 }
@@ -499,10 +504,17 @@ void maciici_state::devsel_w(uint8_t devsel)
 static INPUT_PORTS_START(maciici)
 INPUT_PORTS_END
 
+static INPUT_PORTS_START(maciisi)
+	PORT_START("config")
+	PORT_CONFNAME(0x01, 0x00, "FPU")
+	PORT_CONFSETTING(0x00, "No FPU")
+	PORT_CONFSETTING(0x01, "FPU Present")
+INPUT_PORTS_END
+
 /***************************************************************************
     MACHINE DRIVERS
 ***************************************************************************/
-void maciici_state::maciici(machine_config &config)
+void maciici_state::maciixi_base(machine_config &config)
 {
 	M68030(config, m_maincpu, 25000000);
 	m_maincpu->set_addrmap(AS_PROGRAM, &maciici_state::maciici_map);
@@ -536,12 +548,11 @@ void maciici_state::maciici(machine_config &config)
 	rs232b.dcd_handler().set(m_scc, FUNC(z80scc_device::dcdb_w));
 	rs232b.cts_handler().set(m_scc, FUNC(z80scc_device::ctsb_w));
 
-	SPEAKER(config, "lspeaker").front_left();
-	SPEAKER(config, "rspeaker").front_right();
+	SPEAKER(config, "speaker", 2).front();
 	ASC(config, m_asc, C15M, asc_device::asc_type::ASC);
 	m_asc->irqf_callback().set(m_rbv, FUNC(rbv_device::asc_irq_w));
-	m_asc->add_route(0, "lspeaker", 1.0);
-	m_asc->add_route(1, "rspeaker", 1.0);
+	m_asc->add_route(0, "speaker", 1.0, 0);
+	m_asc->add_route(1, "speaker", 1.0, 1);
 
 	R65NC22(config, m_via1, C7M / 10);
 	m_via1->readpa_handler().set(FUNC(maciici_state::via_in_a));
@@ -555,8 +566,13 @@ void maciici_state::maciici(machine_config &config)
 	NSCSI_CONNECTOR(config, "scsi:0", mac_scsi_devices, nullptr);
 	NSCSI_CONNECTOR(config, "scsi:1", mac_scsi_devices, nullptr);
 	NSCSI_CONNECTOR(config, "scsi:2", mac_scsi_devices, nullptr);
-	NSCSI_CONNECTOR(config, "scsi:3", mac_scsi_devices, nullptr);
-	NSCSI_CONNECTOR(config, "scsi:4", mac_scsi_devices, "cdrom");
+	NSCSI_CONNECTOR(config, "scsi:3").option_set("cdrom", NSCSI_CDROM_APPLE).machine_config(
+		[](device_t *device)
+		{
+			device->subdevice<cdda_device>("cdda")->add_route(0, "^^speaker", 1.0, 0);
+			device->subdevice<cdda_device>("cdda")->add_route(1, "^^speaker", 1.0, 1);
+		});
+	NSCSI_CONNECTOR(config, "scsi:4", mac_scsi_devices, nullptr);
 	NSCSI_CONNECTOR(config, "scsi:5", mac_scsi_devices, nullptr);
 	NSCSI_CONNECTOR(config, "scsi:6", mac_scsi_devices, "harddisk");
 	NSCSI_CONNECTOR(config, "scsi:7").option_set("ncr5380", NCR53C80).machine_config([this](device_t *device)
@@ -573,16 +589,18 @@ void maciici_state::maciici(machine_config &config)
 	m_scsihelp->timeout_error_callback().set(FUNC(maciici_state::scsi_berr_w));
 
 	SOFTWARE_LIST(config, "hdd_list").set_original("mac_hdd");
+	SOFTWARE_LIST(config, "cd_list").set_original("mac_cdrom").set_filter("MC68030,MC68030_32");
 
 	RAM(config, m_ram);
 	m_ram->set_default_size("2M");
 	m_ram->set_extra_options("8M,32M,64M,96M,128M");
 
+	SOFTWARE_LIST(config, "flop_mac35_orig").set_original("mac_flop_orig");
+	SOFTWARE_LIST(config, "flop_mac35_clean").set_original("mac_flop_clcracked");
 	SOFTWARE_LIST(config, "flop35_list").set_original("mac_flop");
 
-	RBV(config, m_rbv, C15M);
+	RBV(config, m_rbv, 31.3344_MHz_XTAL); // main clock input - additional 30.24MHz and 57.2832MHz pixel clock inputs
 	m_rbv->via6015_callback().set(m_via1, FUNC(via6522_device::write_ca1));
-	m_rbv->via6015_callback().append(FUNC(maciici_state::write_6015));
 	m_rbv->irq_callback().set(FUNC(maciici_state::set_via2_interrupt));
 
 	/* internal ram */
@@ -591,43 +609,73 @@ void maciici_state::maciici(machine_config &config)
 
 	nubus_device &nubus(NUBUS(config, "nubus", 0));
 	nubus.set_space(m_maincpu, AS_PROGRAM);
-	nubus.out_irqc_callback().set(m_rbv, FUNC(rbv_device::slot0_irq_w));
-	nubus.out_irqd_callback().set(m_rbv, FUNC(rbv_device::slot1_irq_w));
-	nubus.out_irqe_callback().set(m_rbv, FUNC(rbv_device::slot2_irq_w));
+	nubus.out_irq9_callback().set(m_rbv, FUNC(rbv_device::slot_irq_w<0x01>));
+	nubus.out_irqa_callback().set(m_rbv, FUNC(rbv_device::slot_irq_w<0x02>));
+	nubus.out_irqb_callback().set(m_rbv, FUNC(rbv_device::slot_irq_w<0x04>));
+	nubus.out_irqc_callback().set(m_rbv, FUNC(rbv_device::slot_irq_w<0x08>));
+	nubus.out_irqd_callback().set(m_rbv, FUNC(rbv_device::slot_irq_w<0x10>));
+	nubus.out_irqe_callback().set(m_rbv, FUNC(rbv_device::slot_irq_w<0x20>));
 
 	NUBUS_SLOT(config, "nbc", "nubus", mac_nubus_cards, nullptr);
 	NUBUS_SLOT(config, "nbd", "nubus", mac_nubus_cards, nullptr);
 	NUBUS_SLOT(config, "nbe", "nubus", mac_nubus_cards, nullptr);
+}
+
+void maciici_state::maciici(machine_config &config)
+{
+	maciixi_base(config);
+
+	ADBMODEM(config, m_adbmodem, C7M);
+	m_adbmodem->via_clock_callback().set(m_via1, FUNC(via6522_device::write_cb1));
+	m_adbmodem->via_data_callback().set(m_via1, FUNC(via6522_device::write_cb2));
+	m_adbmodem->linechange_callback().set(m_macadb, FUNC(macadb_device::adb_linechange_w));
+	m_adbmodem->irq_callback().set(FUNC(maciici_state::adb_irq_w));
+	m_via1->cb2_handler().set(m_adbmodem, FUNC(adbmodem_device::set_via_data));
+	config.set_perfect_quantum(m_maincpu);
 
 	MACADB(config, m_macadb, C15M);
-	m_macadb->set_mcu_mode(false);
-	m_macadb->via_clock_callback().set(m_via1, FUNC(via6522_device::write_cb1));
-	m_macadb->via_data_callback().set(m_via1, FUNC(via6522_device::write_cb2));
-	m_macadb->adb_irq_callback().set(FUNC(maciici_state::adb_irq_w));
+	m_macadb->adb_data_callback().set(m_adbmodem, FUNC(adbmodem_device::set_adb_line));
 }
 
 void maciici_state::maciisi(machine_config &config)
 {
-	maciici(config);
+	maciixi_base(config);
 
 	M68030(config.replace(), m_maincpu, 20000000);
 	m_maincpu->set_addrmap(AS_PROGRAM, &maciici_state::maciici_map);
 	m_maincpu->set_dasm_override(std::function(&mac68k_dasm_override), "mac68k_dasm_override");
 
-	MACADB(config.replace(), m_macadb, C15M);
-
 	m_via1->readpa_handler().set(FUNC(maciici_state::via_in_a_iisi));
 	m_via1->readpb_handler().set(FUNC(maciici_state::via_in_b_iisi));
 	m_via1->writepb_handler().set(FUNC(maciici_state::via_out_b_iisi));
 	m_via1->cb2_handler().set(FUNC(maciici_state::via_out_cb2_iisi));
 
-	EGRET(config, m_egret, EGRET_344S0100);
+	MACADB(config, m_macadb, C15M);
+
+	EGRET(config, m_egret, XTAL(32'768));
+	m_egret->set_default_bios_tag("344s0100");
 	m_egret->reset_callback().set(FUNC(maciici_state::egret_reset_w));
 	m_egret->linechange_callback().set(m_macadb, FUNC(macadb_device::adb_linechange_w));
 	m_egret->via_clock_callback().set(m_via1, FUNC(via6522_device::write_cb1));
 	m_egret->via_data_callback().set(m_via1, FUNC(via6522_device::write_cb2));
 	m_macadb->adb_data_callback().set(m_egret, FUNC(egret_device::set_adb_line));
 	config.set_perfect_quantum(m_maincpu);
+
+	config.device_remove("nbc");
+	config.device_remove("nbd");
+	config.device_remove("nbe");
+	config.device_remove("nubus");
+
+	// TODO: IIsi takes an adapter card that can accept either one SE/30 PDS card or one NuBus card
+	nubus_device &nubus(NUBUS(config, "pds", 0));
+	nubus.set_space(m_maincpu, AS_PROGRAM);
+	nubus.out_irq9_callback().set(m_rbv, FUNC(rbv_device::slot_irq_w<0x01>));
+	nubus.out_irqa_callback().set(m_rbv, FUNC(rbv_device::slot_irq_w<0x02>));
+	nubus.out_irqb_callback().set(m_rbv, FUNC(rbv_device::slot_irq_w<0x04>));
+	nubus.out_irqc_callback().set(m_rbv, FUNC(rbv_device::slot_irq_w<0x08>));
+	nubus.out_irqd_callback().set(m_rbv, FUNC(rbv_device::slot_irq_w<0x10>));
+	nubus.out_irqe_callback().set(m_rbv, FUNC(rbv_device::slot_irq_w<0x20>));
+	NUBUS_SLOT(config, "siexp", "pds", mac_iisi_cards, nullptr);
 }
 
 ROM_START( maciici )
@@ -646,4 +694,4 @@ ROM_END
 } // anonymous namespace
 
 COMP(1989, maciici, 0, 0, maciici, maciici, maciici_state, empty_init, "Apple Computer", "Macintosh IIci", MACHINE_SUPPORTS_SAVE)
-COMP(1990, maciisi, 0, 0, maciisi, maciici, maciici_state, empty_init, "Apple Computer", "Macintosh IIsi", MACHINE_SUPPORTS_SAVE)
+COMP(1990, maciisi, 0, 0, maciisi, maciisi, maciici_state, empty_init, "Apple Computer", "Macintosh IIsi", MACHINE_SUPPORTS_SAVE)
