@@ -3,57 +3,53 @@
 // thanks-to: Fujix
 /**************************************************************************************************
 
-    PC-88VA (c) 1987 NEC
+PC-88VA (c) 1987 NEC
 
-    Here be dragons, a mostly compatible PC-8801 with extra V3 Mode for superset.
+Here be dragons, a mostly compatible PC-8801 with extra V3 Mode for superset.
 
-    preliminary driver by Angelo Salese
-    Special thanks to Fujix for his documentation translation help
+TODO:
+- pc88va (stock version) has two bogus opcodes.
+  One is at 0xf0b15 (0x0f 0xfe), another at 0xf0b31 (br 1000h:0c003h).
+  Latter will make the program flow to jump to lalaland.
+  This also happens if you load a regular V1/V2 game assuming you have FDC PIO properly
+  hooked up, is the first opcode actually a Z80 mode switch?
+- pc88va is also known to have a slightly different banking scheme and
+  regular YM2203 as default sound board.
+- video emulation is lacking many features, cfr. pc88va_v.cpp;
+- keyboard runs on undumped MCU, we currently stick irqs together on
+  selected keys in order to have an easier QoL while testing this.
+- Backport from PC-8801 main map, apply supersets where applicable;
+  \- IDP has EMUL for upd3301
+  \- In emulation mode HW still relies to a i8214, so it bridges thru
+     main ICU in cascaded mode via IRQ7;
+  \- beeper or dac1bit (to be confirmed);
+  \- (other stuff ...)
+- Convert FDC usage to pc88va2_fd_if_device, we also need PIO comms for sorcer anyway;
+- irq dispatch needs to be revisited, too many instances of sound irq failing for example.
+  The current hook-ups aren't legal, V50 core bug?
+- Very inconsistent SW boot behaviours, either down to:
+  \- the current hack in FDC PIO port returning RNG;
+  \- V50 timings;
+  \- FDC;
+- Every PC Engine OS boot tries to write TVRAM ASCII data on every boot to
+  $exxxx ROM region, banking bug?
+- all N88 BASIC entries tries to do stuff with EMM, more banking?
 
-    TODO:
-    - pc88va (stock version) has two bogus opcodes.
-      One is at 0xf0b15 (0x0f 0xfe), another at 0xf0b31 (br 1000h:0c003h).
-      Latter will make the program flow to jump to lalaland.
-      This also happens if you load a regular V1/V2 game assuming you have FDC PIO properly
-      hooked up, is the first opcode actually a Z80 mode switch?
-    - pc88va is also known to have a slightly different banking scheme and
-      regular YM2203 as default sound board.
-    - video emulation is lacking many features, cfr. pc88va_v.cpp;
-    - keyboard runs on undumped MCU, we currently stick irqs together on
-      selected keys in order to have an easier QoL while testing this.
-    - Backport from PC-8801 main map, apply supersets where applicable;
-      \- IDP has EMUL for upd3301
-      \- In emulation mode HW still relies to a i8214, so it bridges thru
-         main ICU in cascaded mode via IRQ7;
-      \- beeper or dac1bit (to be confirmed);
-      \- (other stuff ...)
-    - Convert FDC usage to pc88va2_fd_if_device, we also need PIO comms for sorcer anyway;
-    - irq dispatch needs to be revisited, too many instances of sound irq failing for example.
-      The current hook-ups aren't legal, V50 core bug?
-    - Very inconsistent SW boot behaviours, either down to:
-      \- the current hack in FDC PIO port returning RNG;
-      \- V50 timings;
-      \- FDC;
-    - Every PC Engine OS boot tries to write TVRAM ASCII data on every boot to
-      $exxxx ROM region, banking bug?
-    - all N88 BASIC entries tries to do stuff with EMM, more banking?
-    - Convert SASI from PC-9801 to a shared device, apparently it's same i/f;
-    - Implement bus slot, which should still be PC-8801 EXPansion bus.
+(old notes, to be reordered)
+- fdc "intelligent mode" has 0x7f as irq vector ... 0x7f is ld a,a and it IS NOT correctly
+  hooked up by the current z80 core
+- Fix floppy motor hook-up (floppy believes to be always in even if empty drive);
+- Support for PC8801 compatible mode & PC80S31K (floppy interface);
 
-    (old notes, to be reordered)
-    - fdc "intelligent mode" has 0x7f as irq vector ... 0x7f is ld a,a and it IS NOT correctly
-      hooked up by the current z80 core
-    - Fix floppy motor hook-up (floppy believes to be always in even if empty drive);
-    - Support for PC8801 compatible mode & PC80S31K (floppy interface);
+Notes:
+- hold F8 at POST to bring software dip settings menu, F5 to cycle between pages;
+- PC-88VA-91 is a ROM upgrade kit for a PC-88VA -> VA2/VA3.
+  Has four roms, marked by VAEG as VUROM00.ROM, VUROM08.ROM, VUROM1.ROM, VUDIC.ROM.
 
-    Notes:
-    - hold F8 at POST to bring software dip settings menu
-    - PC-88VA-91 is a ROM upgrade kit for a PC-88VA -> VA2/VA3.
-      Has four roms, marked by VAEG as VUROM00.ROM, VUROM08.ROM, VUROM1.ROM, VUDIC.ROM.
-
-    References:
-    - PC-88VAテクニカルマニュアル
-    - http://www.pc88.gr.jp/vafaq/view.php/articlelist/88va/vafaq
+References:
+- PC-88VAテクニカルマニュアル
+- http://www.pc88.gr.jp/vafaq/view.php/articlelist/88va/vafaq
+- I/O magazine 1987 08 (schematics)
 
 ===================================================================================================
 
@@ -62,15 +58,15 @@ ICU
 irq 0  - 08h - timer 1
 irq 1  - 09h - keyboard irq
 irq 2  - 0Ah - VRTC
-irq 3  - 0Bh - UINT0 (B24)
+irq 3  - 0Bh - UINT0 (B24) C-Bus IR3
 irq 4  - 0Ch - RS-232C
-irq 5  - 0Dh - UINT1 (B25)
-irq 6  - 0Eh - UINT2 (B26)
+irq 5  - 0Dh - UINT1 (B25) C-Bus IR5
+irq 6  - 0Eh - UINT2 (B26) C-Bus IR6
 irq 7  - N/A - Slave (either secondary i8259 or i8214)
 i8259 slave
 irq 8  - 10H - SGP
-irq 9  - 11H - UINT3 (HDD, B27)
-irq 10 - 12H - UINT4 (B28)
+irq 9  - 11H - UINT3 (HDD, B27) C-Bus IR9
+irq 10 - 12H - UINT4 (B28) C-Bus IR10
 irq 11 - 13H - FDC
 irq 12 - 14H - Sound
 irq 13 - 15H - General timer 3 (mouse)
@@ -86,13 +82,16 @@ brk 8Ch AH=02h read calendar clock -> CH = hour, CL = minutes, DH = seconds, DL
 #include "emu.h"
 #include "pc88va.h"
 
+#include "bus/pc98_cbus/options.h"
+
 #include "softlist_dev.h"
 
 #include <iostream>
 #include "utf8.h"
 
-#define LOG_FDC     (1U << 2) // $1b0-$1b2 accesses
-#define LOG_FDC2    (1U << 3) // $1b4-$1b6 accesses (verbose)
+#define LOG_FDC      (1U << 2) // $1b0-$1b2 accesses
+#define LOG_FDC2     (1U << 3) // $1b4-$1b6 accesses (verbose)
+#define LOG_GFXCTRL  (1U << 4) // $5xx accesses
 
 #define VERBOSE (LOG_GENERAL | LOG_FDC)
 //#define LOG_OUTPUT_STREAM std::cout
@@ -101,22 +100,23 @@ brk 8Ch AH=02h read calendar clock -> CH = hour, CL = minutes, DH = seconds, DL
 
 #define LOGFDC(...)      LOGMASKED(LOG_FDC, __VA_ARGS__)
 #define LOGFDC2(...)     LOGMASKED(LOG_FDC2, __VA_ARGS__)
+#define LOGGFXCTRL(...)  LOGMASKED(LOG_GFXCTRL, __VA_ARGS__)
 
 // TODO: verify clocks
-#define MASTER_CLOCK    XTAL(8'000'000) // may be XTAL(31'948'800) / 4? (based on PC-8801 and PC-9801)
-#define FM_CLOCK        (XTAL(31'948'800) / 4) // 3993600
+#define MASTER_CLOCK    (XTAL(31'948'800) / 4) // (based on PC-8801 and PC-9801)
+#define FM_CLOCK        (XTAL(31'948'800) / 4) // 3993600, / 8 for regular pc88va
 
 
 
 uint8_t pc88va_state::kanji_ram_r(offs_t offset)
 {
-	return m_kanjiram[offset];
+	return m_kanji_ram[offset];
 }
 
 // TODO: settings area should be write protected depending on the m_backupram_wp bit, separate from this
 void pc88va_state::kanji_ram_w(offs_t offset, uint8_t data)
 {
-	m_kanjiram[offset] = data;
+	m_kanji_ram[offset] = data;
 	m_gfxdecode->gfx(2)->mark_dirty(offset / 8);
 	m_gfxdecode->gfx(3)->mark_dirty(offset / 32);
 }
@@ -187,6 +187,7 @@ void pc88va_state::rtc_w(offs_t offset, u8 data)
 void pc88va_state::bios_bank_w(offs_t offset, uint16_t data, uint16_t mem_mask)
 {
 	COMBINE_DATA(&m_bank_reg);
+	m_gmsp_view.select(BIT(m_bank_reg, 12));
 
 	/* SMBC */
 	m_sysbank->set_bank((m_bank_reg & 0xf00) >> 8);
@@ -256,12 +257,7 @@ void pc88va_state::sys_port5_w(u8 data)
 
 u8 pc88va_state::sys_port5_r()
 {
-	return m_rstmd | 8;
-}
-
-uint8_t pc88va_state::hdd_status_r()
-{
-	return 0x20;
+	return (m_rstmd ? 1 : 0) | 8;
 }
 
 // TODO: convert to pc80s31k family
@@ -270,7 +266,7 @@ uint8_t pc88va_state::fake_subfdc_r()
 	return machine().rand();
 }
 
-uint8_t pc88va_state::pc88va_fdc_r(offs_t offset)
+uint8_t pc88va_state::fdc_r(offs_t offset)
 {
 	if (!machine().side_effects_disabled())
 		LOGFDC("Unhandled read $%04x\n", (offset << 1) + 0x1b0);
@@ -288,7 +284,7 @@ uint8_t pc88va_state::pc88va_fdc_r(offs_t offset)
 	return 0xff;
 }
 
-TIMER_CALLBACK_MEMBER(pc88va_state::pc88va_fdc_timer)
+TIMER_CALLBACK_MEMBER(pc88va_state::fdc_timer)
 {
 	if(m_xtmask)
 	{
@@ -299,17 +295,13 @@ TIMER_CALLBACK_MEMBER(pc88va_state::pc88va_fdc_timer)
 	m_fdc_timer->adjust(attotime::from_msec(100));
 }
 
-TIMER_CALLBACK_MEMBER(pc88va_state::pc88va_fdc_motor_start_0)
+template <unsigned N>
+TIMER_CALLBACK_MEMBER(pc88va_state::fdc_motor_start)
 {
-	m_fdd[0]->get_device()->mon_w(0);
+	m_fdd[N]->get_device()->mon_w(0);
 }
 
-TIMER_CALLBACK_MEMBER(pc88va_state::pc88va_fdc_motor_start_1)
-{
-	m_fdd[1]->get_device()->mon_w(0);
-}
-
-void pc88va_state::pc88va_fdc_update_ready(floppy_image_device *, int)
+void pc88va_state::fdc_update_ready(floppy_image_device *, int)
 {
 	if (!BIT(m_fdc_ctrl_2, 5))
 		return;
@@ -338,7 +330,7 @@ void pc88va_state::pc88va_fdc_update_ready(floppy_image_device *, int)
 		m_fdc->set_ready_line_connected(1);
 }
 
-void pc88va_state::pc88va_fdc_w(offs_t offset, uint8_t data)
+void pc88va_state::fdc_w(offs_t offset, uint8_t data)
 {
 	switch(offset << 1)
 	{
@@ -378,7 +370,9 @@ void pc88va_state::pc88va_fdc_w(offs_t offset, uint8_t data)
 			//m_fdd[0]->get_device()->ds_w(!BIT(data, 4));
 			//m_fdd[1]->get_device()->ds_w(!BIT(data, 4));
 
-			// TODO: is this correct? sounds more like a controller clock change, while TD1/TD0 should do the rate change
+			// TODO: needs source xtal for 4.8 MHz
+			// 8 MHz just uses MASTER_CLOCK
+			m_fdc->set_unscaled_clock(clk ? 7'987'200 : 4'792'320);
 			m_fdc->set_rate(clk ? 500000 : 250000);
 			break;
 		}
@@ -459,7 +453,7 @@ void pc88va_state::pc88va_fdc_w(offs_t offset, uint8_t data)
 
 			//m_fdd[0]->get_device()->mon_w(!(BIT(data, 5)));
 
-			pc88va_fdc_update_ready(nullptr, 0);
+			fdc_update_ready(nullptr, 0);
 
 			break;
 		}
@@ -512,9 +506,9 @@ TIMER_CALLBACK_MEMBER(pc88va_state::t3_mouse_callback)
 uint8_t pc88va_state::backupram_dsw_r(offs_t offset)
 {
 	if(offset == 0)
-		return m_kanjiram[0x1fc2 / 2] & 0xff;
+		return m_kanji_ram[0x1fc2 / 2] & 0xff;
 
-	return m_kanjiram[0x1fc6 / 2] & 0xff;
+	return m_kanji_ram[0x1fc6 / 2] & 0xff;
 }
 
 // TODO: pc8801_state::port31_w
@@ -554,13 +548,15 @@ void pc88va_state::main_map(address_map &map)
 
 void pc88va_state::sysbank_map(address_map &map)
 {
-	// 0 select bus slot
+	// 0 select C-bus slot
 	// 1 tvram
 	map(0x040000, 0x04ffff).ram().share("tvram");
 	// FIXME: BASIC and pacmana expects to r/w to 0x60000-0x7ffff on loading, assume mirror if not a core bug.
 	map(0x050000, 0x07ffff).ram();
 	// 4 gvram
-	map(0x100000, 0x13ffff).ram().share("gvram");
+	map(0x100000, 0x13ffff).view(m_gmsp_view);
+	m_gmsp_view[0](0x100000, 0x13ffff).rw(FUNC(pc88va_state::gvram_multiplane_r), FUNC(pc88va_state::gvram_multiplane_w));
+	m_gmsp_view[1](0x100000, 0x13ffff).rw(FUNC(pc88va_state::gvram_singleplane_r), FUNC(pc88va_state::gvram_singleplane_w));
 	// 8-9 kanji
 	// Kanji ROM
 	map(0x200000, 0x23ffff).rom().region("kanji", 0x00000);
@@ -582,7 +578,11 @@ void pc88va_state::sgp_map(address_map &map)
 	map(0x140000, 0x14ffff).rom().region("kanji", 0x40000);
 	map(0x150000, 0x153fff).rw(FUNC(pc88va_state::kanji_ram_r),FUNC(pc88va_state::kanji_ram_w));
 	map(0x180000, 0x18ffff).ram().share("tvram");
-	map(0x200000, 0x23ffff).ram().share("gvram");
+	// Assume just raw writes to GVRAM
+	map(0x200000, 0x23ffff).lrw8(
+		NAME([this] (offs_t offset) { return m_gvram[offset]; }),
+		NAME([this] (offs_t offset, u8 data) { m_gvram[offset] = data; })
+	);
 }
 
 // TODO: I/O 0x00xx is almost same as pc8801
@@ -605,8 +605,15 @@ void pc88va_state::io_map(address_map &map)
 //  map(0x0070, 0x0070) ? (*)
 //  map(0x0071, 0x0071) Expansion ROM select (*)
 //  map(0x0078, 0x0078) Memory offset increment (*)
-//  map(0x0080, 0x0081) HDD related
-	map(0x0082, 0x0082).r(FUNC(pc88va_state::hdd_status_r));// HDD control, byte access 7-0
+	// SASI hole
+	map(0x0080, 0x0083).lrw16(
+		NAME([this] (offs_t offset, u16 mem_mask) {
+			return m_cbus_root->io_r(offset + 0x40, mem_mask);
+		}),
+		NAME([this] (offs_t offset, u16 data, u16 mem_mask) {
+			m_cbus_root->io_w(offset + 0x40, data, mem_mask);
+		})
+	);
 //  map(0x00bc, 0x00bf) d8255 1
 //  map(0x00e2, 0x00e3) Expansion RAM selection (*)
 //  map(0x00e4, 0x00e4) 8214 IRQ control (*)
@@ -625,7 +632,7 @@ void pc88va_state::io_map(address_map &map)
 //  map(0x0124, 0x0125) ? (related to Transparent Color of Graphic Screen 0)
 //  map(0x0126, 0x0127) ? (related to Transparent Color of Graphic Screen 1)
 	map(0x012e, 0x012f).w(FUNC(pc88va_state::text_transpen_w));
-//  map(0x0130, 0x0137) Picture Mask Parameter (global cliprect, olteus gameplay)
+	map(0x0130, 0x0137).w(FUNC(pc88va_state::picture_mask_w));
 	map(0x0142, 0x0142).rw(FUNC(pc88va_state::idp_status_r), FUNC(pc88va_state::idp_command_w)); //Text Controller (IDP) - (R) Status (W) command
 	map(0x0146, 0x0146).w(FUNC(pc88va_state::idp_param_w)); //Text Controller (IDP) - (R/W) Parameter
 	map(0x0148, 0x0148).w(FUNC(pc88va_state::text_control_1_w));
@@ -640,7 +647,7 @@ void pc88va_state::io_map(address_map &map)
 //  map(0x0158, 0x0159) Interruption Mode Modification (strobe), changes i8214 mode to i8259, cannot be changed back
 //  map(0x015c, 0x015f) NMI mask port (strobe port)
 //  map(0x0160, 0x016f) V50 DMAC
-//  map(0x0180, 0x0180) read by Olteus
+//  map(0x0180, 0x0180) read by olteus and micromus (?)
 	map(0x0184, 0x0187).rw("pic8259_slave", FUNC(pic8259_device::read), FUNC(pic8259_device::write)).umask16(0x00ff);
 //  map(0x0188, 0x018b) V50 ICU
 //  map(0x0190, 0x0191) System Port 5
@@ -650,7 +657,7 @@ void pc88va_state::io_map(address_map &map)
 	map(0x019a, 0x019b).w(FUNC(pc88va_state::backupram_wp_0_w)); //Backup RAM write permission
 //  map(0x01a0, 0x01a7) V50 TCU
 	map(0x01a8, 0x01a8).w(FUNC(pc88va_state::timer3_ctrl_reg_w)); // General-purpose timer 3 control port
-	map(0x01b0, 0x01b7).rw(FUNC(pc88va_state::pc88va_fdc_r), FUNC(pc88va_state::pc88va_fdc_w)).umask16(0x00ff); // FDC related (765)
+	map(0x01b0, 0x01b7).rw(FUNC(pc88va_state::fdc_r), FUNC(pc88va_state::fdc_w)).umask16(0x00ff); // FDC related (765)
 	map(0x01b8, 0x01bb).m(m_fdc, FUNC(upd765a_device::map)).umask16(0x00ff);
 //  map(0x01c0, 0x01c1) keyboard scan code, polled thru IRQ1 ...
 	map(0x01c1, 0x01c1).lr8(NAME([this] () { return m_keyb.data; }));
@@ -663,24 +670,165 @@ void pc88va_state::io_map(address_map &map)
 
 	map(0x0500, 0x0507).m(m_sgp, FUNC(pc88va_sgp_device::sgp_io));
 	// GVRAM multiplane access regs (ROP section)
-//  map(0x0510, 0x0510) AACC extend access mode
-//  map(0x0512, 0x0512) GMAP block switch
-//  map(0x0514, 0x0514) XRPMn plane readback select
-//  map(0x0516, 0x0516) XWPMn plane write select
-//  map(0x0518, 0x0518) multiplane enable
-//  map(0x0520, 0x0527).umask16(0x00ff) extended access bit comparison
+	// TODO: register are locked with GMSP = 1
+	map(0x0510, 0x0510).lrw8(
+		NAME([this] (offs_t offset) {
+			LOGGFXCTRL("AACC extend access mode R\n");
+			return m_multiplane.aacc;
+		}),
+		NAME([this] (offs_t offset, u8 data) {
+			m_multiplane.aacc = !!BIT(data, 0);
+			LOGGFXCTRL("AACC extend access mode W %02x\n", data);
+		})
+	);
+	map(0x0512, 0x0512).lrw8(
+		NAME([this] (offs_t offset) {
+			LOGGFXCTRL("GMAP block switch R\n");
+			return m_multiplane.gmap;
+		}),
+		NAME([this] (offs_t offset, u8 data) {
+			LOGGFXCTRL("GMAP block switch W %02x\n", data);
+			m_multiplane.gmap = !!BIT(data, 0);
+		})
+	);
+	map(0x0514, 0x0514).lrw8(
+		NAME([this] (offs_t offset) {
+			LOGGFXCTRL("XRPMn plane readback select R\n");
+			return m_multiplane.xrpm | 0xf0;
+		}),
+		NAME([this] (offs_t offset, u8 data) {
+			LOGGFXCTRL("XRPMn plane readback select W %02x\n", data);
+			m_multiplane.xrpm = data & 0xf;
+		})
+	);
+	map(0x0516, 0x0516).lrw8(
+		NAME([this] (offs_t offset) {
+			LOGGFXCTRL("XWPMn plane write select R\n");
+			return m_multiplane.xwpm | 0xf0;
+		}),
+		NAME([this] (offs_t offset, u8 data) {
+			LOGGFXCTRL("XWPMn plane write select W %02x\n", data);
+			m_multiplane.xwpm = data & 0xf;
+		})
+	);
+	map(0x0518, 0x0518).lrw8(
+		NAME([this] (offs_t offset) {
+			// TODO: rbusy reads (bit 7)
+			return (m_multiplane.cmpen << 5) | (m_multiplane.wss << 3) | (m_multiplane.pmod << 0);
+		}),
+		NAME([this] (offs_t offset, u8 data) {
+			LOGGFXCTRL("Multiplane Mode W %02x\n", data);
+			// PMOD bit 2 1 -> 0 transitions resets pattern pointers
+			if (BIT(m_multiplane.pmod, 2) && !BIT(data, 2))
+			{
+				m_multiplane.prrp = 0;
+				m_multiplane.prwp = 0;
+			}
+
+			m_multiplane.cmpen = !!BIT(data, 5);
+			m_multiplane.wss = (data >> 3) & 3;
+			m_multiplane.pmod = (data >> 0) & 7;
+		})
+	);
+	map(0x0520, 0x0527).umask16(0x00ff).lrw8(
+		NAME([this] (offs_t offset) {
+			LOGGFXCTRL("CMPR extended access bit comparison R\n");
+			return m_multiplane.cmpr[offset];
+		}),
+		NAME([this] (offs_t offset, u8 data) {
+			LOGGFXCTRL("CMPR extended access bit comparison W %02x\n", data);
+			m_multiplane.cmpr[offset] = data;
+		})
+	);
 //  map(0x0528, 0x0528) extended access plane comparison
-//  map(0x0530, 0x0537).umask16(0x00ff) extended access pattern low byte
-//  map(0x0540, 0x0547).umask16(0x00ff) extended access pattern high byte
-//  map(0x0550, 0x0550) PRRPn plane pattern usage start byte on read
-//  map(0x0552, 0x0552) PRWPn plane pattern usage start byte on write
-//  map(0x0560, 0x0567).umask16(0x00ff) ROP plane code
+	map(0x0530, 0x0537).umask16(0x00ff).lrw8(
+		NAME([this] (offs_t offset) {
+			LOGGFXCTRL("Multiplane PATRL%d R\n", offset);
+			return m_multiplane.patr[offset][0];
+		}),
+		NAME([this] (offs_t offset, u8 data) {
+			LOGGFXCTRL("Multiplane PATRL%d W %02x\n", offset, data);
+			m_multiplane.patr[offset][0] = data;
+		})
+	);
+	map(0x0540, 0x0547).umask16(0x00ff).lrw8(
+		NAME([this] (offs_t offset) {
+			LOGGFXCTRL("Multiplane PATRH%d R\n", offset);
+			return m_multiplane.patr[offset][1];
+		}),
+		NAME([this] (offs_t offset, u8 data) {
+			LOGGFXCTRL("Multiplane PATRH%d W %02x\n", offset, data);
+			m_multiplane.patr[offset][1] = data;
+		})
+	);
+	map(0x0550, 0x0550).umask16(0x00ff).lrw8(
+		NAME([this] (offs_t offset) {
+			return m_multiplane.prrp | 0xf0;
+		}),
+		NAME([this] (offs_t offset, u8 data) {
+			LOGGFXCTRL("PRRPn plane pattern usage start byte on read %02x\n", data);
+			m_multiplane.prrp = data & 0xf;
+		})
+	);
+	map(0x0552, 0x0552).umask16(0x00ff).lrw8(
+		NAME([this] (offs_t offset) {
+			return m_multiplane.prwp | 0xf0;
+		}),
+		NAME([this] (offs_t offset, u8 data) {
+			LOGGFXCTRL("PRWPn plane pattern usage start byte on write %02x\n", data);
+			m_multiplane.prwp = data & 0xf;
+		})
+	);
+	map(0x0560, 0x0567).umask16(0x00ff).lrw8(
+		NAME([this] (offs_t offset) {
+			return m_multiplane.rop[offset];
+		}),
+		NAME([this] (offs_t offset, u8 data) {
+			LOGGFXCTRL("Multiplane ROP %d W %02x\n", offset, data);
+			m_multiplane.rop[offset] = data;
+		})
+	);
 	// GVRAM single plane access regs
-//  map(0x0580, 0x0580) single plane enable
-//  map(0x0590, 0x0593) GVRAM pattern register settings
-//  map(0x05a0, 0x05a3) ROP plane code
+	// TODO: register are locked with GMSP = 0
+	map(0x0580, 0x0580).lrw8(
+		NAME([this] (offs_t offset) {
+			// TODO: rbusy reads (bit 7)
+			return (m_singleplane.wss << 3);
+		}),
+		NAME([this] (offs_t offset, u8 data) {
+			LOGGFXCTRL("Singleplane Mode W %02x\n", data);
+			m_singleplane.wss = (data >> 3) & 3;
+		})
+	);
+	map(0x0590, 0x0593).umask16(0x00ff).lrw8(
+		NAME([this] (offs_t offset) {
+			LOGGFXCTRL("Singleplane PATR%d R\n", offset);
+			return m_singleplane.patr[offset];
+		}),
+		NAME([this] (offs_t offset, u8 data) {
+			LOGGFXCTRL("Singleplane PATR%d W %02x\n", offset, data);
+			m_singleplane.patr[offset] = data;
+		})
+	);
+	map(0x05a0, 0x05a3).umask16(0x00ff).lrw8(
+		NAME([this] (offs_t offset) {
+			return m_singleplane.rop[offset];
+		}),
+		NAME([this] (offs_t offset, u8 data) {
+			LOGGFXCTRL("Singleplane ROP %d W %02x\n", offset, data);
+			m_singleplane.rop[offset] = data;
+		})
+	);
 
 //  map(0x1000, 0xfeff) PC-88VA expansion boards
+	map(0x1000, 0xfeff).lrw16(
+		NAME([this] (offs_t offset, u16 mem_mask) {
+			return m_cbus_root->io_r(offset + 0x800, mem_mask);
+		}),
+		NAME([this] (offs_t offset, u16 data, u16 mem_mask) {
+			m_cbus_root->io_w(offset + 0x800, data, mem_mask);
+		})
+	);
 //  map(0xe2d2, 0xe2d2) MIDI status in micromus
 //  map(0xff00, 0xffff).noprw(); // CPU internal use
 }
@@ -713,7 +861,7 @@ INPUT_CHANGED_MEMBER(pc88va_state::key_stroke)
 }
 
 #define VA_PORT_SCAN(_scancode_) \
-	PORT_CHANGED_MEMBER(DEVICE_SELF, pc88va_state, key_stroke, _scancode_)
+	PORT_CHANGED_MEMBER(DEVICE_SELF, FUNC(pc88va_state::key_stroke), _scancode_)
 
 static INPUT_PORTS_START( pc88va )
 	PORT_START("KEY0")
@@ -952,8 +1100,8 @@ static const gfx_layout pc88va_kanji_16x16 =
 static GFXDECODE_START( gfx_pc88va )
 	GFXDECODE_ENTRY( "kanji",   0x00000, pc88va_chars_8x8,    0, 16 )
 	GFXDECODE_ENTRY( "kanji",   0x00000, pc88va_chars_16x16,  0, 16 )
-	GFXDECODE_ENTRY( nullptr,   0x00000, pc88va_kanji_8x8,    0, 16 )
-	GFXDECODE_ENTRY( nullptr,   0x00000, pc88va_kanji_16x16,  0, 16 )
+	GFXDECODE_RAM(   nullptr,   0x00000, pc88va_kanji_8x8,    0, 16 )
+	GFXDECODE_RAM(   nullptr,   0x00000, pc88va_kanji_16x16,  0, 16 )
 GFXDECODE_END
 
 
@@ -1026,7 +1174,7 @@ TIMER_DEVICE_CALLBACK_MEMBER(pc88va_state::vrtc_irq)
 	}
 }
 
-WRITE_LINE_MEMBER( pc88va_state::fdc_irq )
+void pc88va_state::fdc_irq(int state)
 {
 	if(m_fdc_mode && state)
 	{
@@ -1035,7 +1183,7 @@ WRITE_LINE_MEMBER( pc88va_state::fdc_irq )
 	}
 }
 
-WRITE_LINE_MEMBER(pc88va_state::int4_irq_w)
+void pc88va_state::int4_irq_w(int state)
 {
 	bool irq_state = m_sound_irq_enable & state;
 
@@ -1048,7 +1196,7 @@ WRITE_LINE_MEMBER(pc88va_state::int4_irq_w)
 	m_sound_irq_pending = state;
 }
 
-WRITE_LINE_MEMBER( pc88va_state::tc_w )
+void pc88va_state::tc_w(int state)
 {
 	m_fdc->tc_w(state);
 }
@@ -1070,11 +1218,11 @@ void pc88va_state::machine_start()
 	m_rtc->cs_w(1);
 	m_rtc->oe_w(1);
 
-	m_fdc_timer = timer_alloc(FUNC(pc88va_state::pc88va_fdc_timer), this);
+	m_fdc_timer = timer_alloc(FUNC(pc88va_state::fdc_timer), this);
 	m_fdc_timer->adjust(attotime::never);
 
-	m_motor_start_timer[0] = timer_alloc(FUNC(pc88va_state::pc88va_fdc_motor_start_0), this);
-	m_motor_start_timer[1] = timer_alloc(FUNC(pc88va_state::pc88va_fdc_motor_start_1), this);
+	m_motor_start_timer[0] = timer_alloc(FUNC(pc88va_state::fdc_motor_start<0>), this);
+	m_motor_start_timer[1] = timer_alloc(FUNC(pc88va_state::fdc_motor_start<1>), this);
 	m_motor_start_timer[0]->adjust(attotime::never);
 	m_motor_start_timer[1]->adjust(attotime::never);
 
@@ -1084,11 +1232,11 @@ void pc88va_state::machine_start()
 	floppy_image_device *floppy;
 	floppy = m_fdd[0]->get_device();
 	if(floppy)
-		floppy->setup_ready_cb(floppy_image_device::ready_cb(&pc88va_state::pc88va_fdc_update_ready, this));
+		floppy->setup_ready_cb(floppy_image_device::ready_cb(&pc88va_state::fdc_update_ready, this));
 
 	floppy = m_fdd[1]->get_device();
 	if(floppy)
-		floppy->setup_ready_cb(floppy_image_device::ready_cb(&pc88va_state::pc88va_fdc_update_ready, this));
+		floppy->setup_ready_cb(floppy_image_device::ready_cb(&pc88va_state::fdc_update_ready, this));
 
 	m_fdd[0]->get_device()->set_rpm(300);
 	m_fdd[1]->get_device()->set_rpm(300);
@@ -1112,6 +1260,7 @@ void pc88va_state::machine_reset()
 
 	m_fdc_mode = 0;
 	m_xtmask = false;
+	m_fdc->set_unscaled_clock(4'792'320);
 
 	// shinraba never write to port $32,
 	// and it expects that the sound irq actually runs otherwise it enters in debug mode
@@ -1120,6 +1269,29 @@ void pc88va_state::machine_reset()
 	m_sound_irq_pending = false;
 }
 
+// NOTE: PC-88VA implementation omits some C-Bus lines compared to PC-98.
+// - doesn't have ir12 and ir13, i.e. covers INT0 to INT4 only
+// - no /CPUKILL pin support
+// cfr. schematics pg. 260, "external bus, videoboard connector"
+void pc88va_state::pc88va_cbus(machine_config &config)
+{
+	PC98_CBUS_ROOT(config, m_cbus_root, 0);
+	m_cbus_root->int_cb<0>().set_inputline(m_maincpu, INPUT_LINE_IRQ3);
+	m_cbus_root->int_cb<1>().set_inputline(m_maincpu, INPUT_LINE_IRQ5);
+	m_cbus_root->int_cb<2>().set_inputline(m_maincpu, INPUT_LINE_IRQ6);
+	m_cbus_root->int_cb<3>().set("pic8259_slave", FUNC(pic8259_device::ir1_w));
+	// TODO: or ir3_w?
+	m_cbus_root->int_cb<4>().set("pic8259_slave", FUNC(pic8259_device::ir2_w));
+	m_cbus_root->drq_cb<0>().set(m_maincpu, FUNC(v50_device::dreq_w<0>)).invert();
+	m_maincpu->in_ior_cb<0>().set([this] () { return m_cbus_root->dack_r(0); });
+	m_maincpu->out_iow_cb<0>().set([this] (u8 data) { return m_cbus_root->dack_w(0, data); });
+
+	// should be 3 slots for each iteration here
+	PC98_CBUS_SLOT(config, "cbus0", 0, "cbus_root", pc88va_cbus_devices, nullptr);
+	PC98_CBUS_SLOT(config, "cbus1", 0, "cbus_root", pc88va_cbus_devices, nullptr);
+	PC98_CBUS_SLOT(config, "cbus2", 0, "cbus_root", pc88va_cbus_devices, nullptr);
+}
+
 void pc88va_state::pc88va(machine_config &config)
 {
 	V50(config, m_maincpu, MASTER_CLOCK); // μPD9002, aka V50 + μPD70008AC (for PC8801 compatibility mode) in place of 8080
@@ -1138,6 +1310,8 @@ void pc88va_state::pc88va(machine_config &config)
 	m_maincpu->in_memr_cb().set([this] (offs_t offset) { return m_maincpu->space(AS_PROGRAM).read_byte(offset); });
 	m_maincpu->out_memw_cb().set([this] (offs_t offset, u8 data) { m_maincpu->space(AS_PROGRAM).write_byte(offset, data); });
 
+	pc88va_cbus(config);
+
 	// TODO: pc80s31k here
 
 	SCREEN(config, m_screen, SCREEN_TYPE_RASTER);
@@ -1163,25 +1337,22 @@ void pc88va_state::pc88va(machine_config &config)
 	m_pic2->out_int_callback().set_inputline(m_maincpu, INPUT_LINE_IRQ7);
 	m_pic2->in_sp_callback().set_constant(0);
 
-	UPD765A(config, m_fdc, 4000000, true, true);
+	// switchable between 8 and 4.8 MHz
+	UPD765A(config, m_fdc, 4'792'320, true, true);
 	m_fdc->intrq_wr_callback().set(FUNC(pc88va_state::fdc_irq));
 	m_fdc->drq_wr_callback().set(m_maincpu, FUNC(v50_device::dreq_w<2>));
 	FLOPPY_CONNECTOR(config, m_fdd[0], pc88va_floppies, "525hd", pc88va_state::floppy_formats).enable_sound(true);
 	FLOPPY_CONNECTOR(config, m_fdd[1], pc88va_floppies, "525hd", pc88va_state::floppy_formats).enable_sound(true);
 
-	// TODO: set pc98 compatible
-	// Needs a MS-Engine disk dump first, that applies an overlay on PC Engine OS so that it can run PC-98 software
-	SOFTWARE_LIST(config, "disk_list").set_original("pc88va");
-
 	UPD4990A(config, m_rtc);
 
 	ADDRESS_MAP_BANK(config, "sysbank").set_map(&pc88va_state::sysbank_map).set_options(ENDIANNESS_LITTLE, 16, 22, 0x40000);
 
 	MSX_GENERAL_PURPOSE_PORT(config, m_mouse_port, msx_general_purpose_port_devices, "joystick");
 
-	SPEAKER(config, m_lspeaker).front_left();
-	SPEAKER(config, m_rspeaker).front_right();
+	SPEAKER(config, m_speaker, 2).front();
 
+	// TODO: YM2203 for vanilla pc88va
 	// PC-88VA-12 "Sound Board II", YM2608B
 	YM2608(config, m_opna, FM_CLOCK);
 	m_opna->set_addrmap(0, &pc88va_state::opna_map);
@@ -1190,10 +1361,15 @@ void pc88va_state::pc88va(machine_config &config)
 	m_opna->port_b_read_callback().set(FUNC(pc88va_state::opn_portb_r));
 	m_opna->port_b_write_callback().set(FUNC(pc88va_state::opn_portb_w));
 	// TODO: per-channel mixing is unconfirmed
-	m_opna->add_route(0, m_lspeaker, 0.25);
-	m_opna->add_route(0, m_rspeaker, 0.25);
-	m_opna->add_route(1, m_lspeaker, 0.75);
-	m_opna->add_route(2, m_rspeaker, 0.75);
+	m_opna->add_route(0, m_speaker, 0.25, 0);
+	m_opna->add_route(0, m_speaker, 0.25, 1);
+	m_opna->add_route(1, m_speaker, 0.50, 0);
+	m_opna->add_route(2, m_speaker, 0.50, 1);
+
+	// TODO: set pc98 compatible
+	// Needs a MS-Engine disk dump first, that applies an overlay on PC Engine OS so that it can run PC-98 software
+	SOFTWARE_LIST(config, "disk_list").set_original("pc88va");
+	SOFTWARE_LIST(config, "flop_generic_list").set_compatible("generic_flop_525").set_filter("pc88va");
 }
 
 
