@@ -37,6 +37,8 @@
 #include "emupal.h"
 #include "screen.h"
 
+#include "formats/pk8020_dsk.h"
+
 
 #define LOG_BANK    (1U << 1)
 
@@ -80,8 +82,7 @@ private:
 	void memory_write_byte(offs_t offset, uint8_t data);
 	uint8_t io_read_byte(offs_t offset);
 	void io_write_byte(offs_t offset, uint8_t data);
-	DECLARE_WRITE_LINE_MEMBER(busreq_w);
-	DECLARE_WRITE_LINE_MEMBER(tc_w);
+	void tc_w(int state);
 	void rt1715_floppy_enable(uint8_t data);
 	uint8_t k7658_led1_r();
 	uint8_t k7658_led2_r();
@@ -93,21 +94,23 @@ private:
 	void rt1715w_krfd_w(uint8_t data);
 	void rt1715_palette(palette_device &palette) const;
 	I8275_DRAW_CHARACTER_MEMBER(crtc_display_pixels);
-	DECLARE_WRITE_LINE_MEMBER(crtc_drq_w);
+	void crtc_drq_w(int state);
 
-	void k7658_io(address_map &map);
-	void k7658_mem(address_map &map);
-	void rt1715_base_io(address_map &map);
-	void rt1715_io(address_map &map);
-	void rt1715w_io(address_map &map);
-	void rt1715_mem(address_map &map);
-	void rt1715w_mem(address_map &map);
+	void k7658_io(address_map &map) ATTR_COLD;
+	void k7658_mem(address_map &map) ATTR_COLD;
+	void rt1715_base_io(address_map &map) ATTR_COLD;
+	void rt1715_io(address_map &map) ATTR_COLD;
+	void rt1715w_io(address_map &map) ATTR_COLD;
+	void rt1715_mem(address_map &map) ATTR_COLD;
+	void rt1715w_mem(address_map &map) ATTR_COLD;
 
 	DECLARE_MACHINE_START(rt1715);
 	DECLARE_MACHINE_RESET(rt1715);
 	DECLARE_MACHINE_START(rt1715w);
 	DECLARE_MACHINE_RESET(rt1715w);
 
+	static void floppy_formats(format_registration &fr);
+
 	required_device<z80_device> m_maincpu;
 	required_device<ram_device> m_ram;
 	required_device<z80sio_device> m_sio0;
@@ -146,8 +149,8 @@ void rt1715_state::rt1715w_floppy_motor(uint8_t data)
 {
 	LOG("%s: rt1715w_floppy_motor %02x\n", machine().describe_context(), data);
 
-	if (m_floppy[0]->get_device()) m_floppy[0]->get_device()->mon_w(data & 0x80 ? 1 : 0);
-	if (m_floppy[1]->get_device()) m_floppy[1]->get_device()->mon_w(data & 0x08 ? 1 : 0);
+	if (m_floppy[0]->get_device()) m_floppy[0]->get_device()->mon_w(BIT(data, 7));
+	if (m_floppy[1]->get_device()) m_floppy[1]->get_device()->mon_w(BIT(data, 3));
 }
 
 void rt1715_state::rt1715w_krfd_w(uint8_t data)
@@ -156,7 +159,7 @@ void rt1715_state::rt1715w_krfd_w(uint8_t data)
 	m_krfd = data;
 }
 
-WRITE_LINE_MEMBER(rt1715_state::tc_w)
+void rt1715_state::tc_w(int state)
 {
 	m_fdc->tc_w(state & BIT(m_krfd, 7));
 }
@@ -401,18 +404,11 @@ void rt1715_state::io_write_byte(offs_t offset, uint8_t data)
 	prog_space.write_byte(offset, data);
 }
 
-WRITE_LINE_MEMBER(rt1715_state::busreq_w)
-{
-	// since our Z80 has no support for BUSACK, we assume it is granted immediately
-	m_maincpu->set_input_line(INPUT_LINE_HALT, state);
-	m_dma->bai_w(state); // tell dma that bus has been granted
-}
-
 /***************************************************************************
     VIDEO EMULATION
 ***************************************************************************/
 
-WRITE_LINE_MEMBER(rt1715_state::crtc_drq_w)
+void rt1715_state::crtc_drq_w(int state)
 {
 	if (state)
 	{
@@ -423,15 +419,18 @@ WRITE_LINE_MEMBER(rt1715_state::crtc_drq_w)
 
 I8275_DRAW_CHARACTER_MEMBER(rt1715_state::crtc_display_pixels)
 {
+	using namespace i8275_attributes;
+
 	rgb_t const *const palette = m_palette->palette()->entry_list_raw();
-	u8 gfx = (lten) ? 0xff : 0;
+	u8 gfx = BIT(attrcode, LTEN) ? 0xff : 0;
 
-	if (!vsp)
-		gfx = m_p_chargen[((gpa & 1) << 11) | (linecount << 7) | charcode];
+	if (!BIT(attrcode, VSP))
+		gfx = m_p_chargen[(BIT(attrcode, GPA0) ? 0x800 : 0) | (linecount << 7) | charcode];
 
-	if (rvv)
+	if (BIT(attrcode, RVV))
 		gfx ^= 0xff;
 
+	bool hlgt = BIT(attrcode, HLGT);
 	for (u8 i=0; i<8; i++)
 		bitmap.pix(y, x + i) = palette[BIT(gfx, 7-i) ? (hlgt ? 2 : 1) : 0];
 }
@@ -612,7 +611,7 @@ static INPUT_PORTS_START( k7658 )
 	PORT_BIT( 0x10, IP_ACTIVE_HIGH, IPT_KEYBOARD ) PORT_CODE(KEYCODE_Z) PORT_CHAR('z') PORT_CHAR('Z')
 	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_KEYBOARD ) PORT_CODE(KEYCODE_CLOSEBRACE) PORT_CHAR(']') PORT_CHAR('}')
 	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_KEYBOARD ) PORT_CODE(KEYCODE_INSERT) PORT_CHAR(UCHAR_MAMEKEY(INSERT))
-	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_KEYBOARD ) PORT_CODE(KEYCODE_TAB) PORT_CHAR('\t')
+	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_KEYBOARD ) PORT_CODE(KEYCODE_UP) PORT_CHAR(UCHAR_MAMEKEY(UP))
 
 	PORT_START("row_60")
 	// D07 A17 E07 B17 B07 C07 D17 C17
@@ -688,7 +687,7 @@ static INPUT_PORTS_START( k7658 )
 	PORT_BIT( 0x08, IP_ACTIVE_HIGH, IPT_KEYBOARD ) PORT_CODE(KEYCODE_LEFT) PORT_CHAR(UCHAR_MAMEKEY(LEFT))
 	PORT_BIT( 0x10, IP_ACTIVE_HIGH, IPT_KEYBOARD ) PORT_CODE(KEYCODE_N) PORT_CHAR('n') PORT_CHAR('N')
 	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_KEYBOARD ) PORT_CODE(KEYCODE_H) PORT_CHAR('h') PORT_CHAR('H')
-	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_KEYBOARD ) PORT_CODE(KEYCODE_UP) PORT_CHAR(UCHAR_MAMEKEY(UP))
+	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_KEYBOARD ) PORT_CODE(KEYCODE_TAB) PORT_CHAR('\t') // FIXME
 	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_KEYBOARD ) PORT_NAME("linker Rand")
 INPUT_PORTS_END
 
@@ -718,10 +717,17 @@ static const z80_daisy_config rt1715_daisy_chain[] =
 
 static const z80_daisy_config rt1715w_daisy_chain[] =
 {
+	{ "ctc2" },
 	{ "sio0" },
 	{ nullptr }
 };
 
+void rt1715_state::floppy_formats(format_registration &fr)
+{
+	fr.add_mfm_containers();
+	fr.add(FLOPPY_PK8020_FORMAT);
+}
+
 static void rt1715w_floppies(device_slot_interface &device)
 {
 	device.option_add("525qd", FLOPPY_525_QD);
@@ -746,7 +752,7 @@ void rt1715_state::rt1715(machine_config &config)
 	/* video hardware */
 	SCREEN(config, m_screen, SCREEN_TYPE_RASTER);
 	m_screen->set_screen_update("i8275", FUNC(i8275_device::screen_update));
-	m_screen->set_raw(13.824_MHz_XTAL, 864, 0, 624, 320, 0, 300); // ?
+	m_screen->set_raw(13.824_MHz_XTAL, 864, 0, 640, 320, 0, 288);
 
 	GFXDECODE(config, "gfxdecode", "palette", gfx_rt1715);
 	PALETTE(config, "palette", FUNC(rt1715_state::rt1715_palette), 3);
@@ -763,6 +769,7 @@ void rt1715_state::rt1715(machine_config &config)
 	m_sio0->out_dtrb_callback().set(m_rs232, FUNC(rs232_port_device::write_dtr));
 	m_sio0->out_rtsb_callback().set(m_rs232, FUNC(rs232_port_device::write_rts));
 
+	// 1715W: A5, IEO output NC
 	Z80CTC(config, m_ctc0, 15.9744_MHz_XTAL / 4);
 	m_ctc0->zc_callback<0>().set(m_sio0, FUNC(z80sio_device::txca_w));
 	m_ctc0->zc_callback<2>().set(m_sio0, FUNC(z80sio_device::rxtxcb_w));
@@ -793,6 +800,7 @@ void rt1715_state::rt1715w(machine_config &config)
 	m_maincpu->set_addrmap(AS_PROGRAM, &rt1715_state::rt1715w_mem);
 	m_maincpu->set_addrmap(AS_IO, &rt1715_state::rt1715w_io);
 	m_maincpu->set_daisy_config(rt1715w_daisy_chain);
+	m_maincpu->busack_cb().set(m_dma, FUNC(z80dma_device::bai_w));
 
 	MCFG_MACHINE_START_OVERRIDE(rt1715_state, rt1715w)
 	MCFG_MACHINE_RESET_OVERRIDE(rt1715_state, rt1715w)
@@ -802,21 +810,25 @@ void rt1715_state::rt1715w(machine_config &config)
 
 	m_crtc->drq_wr_callback().set(FUNC(rt1715_state::crtc_drq_w));
 
-	// operates in polled mode
+	// FIXME interrupt output NC, but vector 0x14 set to ISR 0xf6bc, find source
 	I8272A(config, m_fdc, 8'000'000 / 4, false);
 	m_fdc->drq_wr_callback().set(m_dma, FUNC(z80dma_device::rdy_w)).invert();
-	FLOPPY_CONNECTOR(config, "i8272:0", rt1715w_floppies, "525qd", floppy_image_device::default_mfm_floppy_formats);
-	FLOPPY_CONNECTOR(config, "i8272:1", rt1715w_floppies, "525qd", floppy_image_device::default_mfm_floppy_formats);
+	FLOPPY_CONNECTOR(config, "i8272:0", rt1715w_floppies, "525qd", rt1715_state::floppy_formats);
+	FLOPPY_CONNECTOR(config, "i8272:1", rt1715w_floppies, "525qd", rt1715_state::floppy_formats);
 
 	Z80DMA(config, m_dma, 15.9744_MHz_XTAL / 4);
-	m_dma->out_busreq_callback().set(FUNC(rt1715_state::busreq_w));
+	m_dma->out_busreq_callback().set_inputline(m_maincpu, Z80_INPUT_LINE_BUSRQ);
 	m_dma->out_int_callback().set(FUNC(rt1715_state::tc_w));
 	m_dma->in_mreq_callback().set(FUNC(rt1715_state::memory_read_byte));
 	m_dma->out_mreq_callback().set(FUNC(rt1715_state::memory_write_byte));
 	m_dma->in_iorq_callback().set(FUNC(rt1715_state::io_read_byte));
 	m_dma->out_iorq_callback().set(FUNC(rt1715_state::io_write_byte));
 
+	// A4
 	Z80CTC(config, m_ctc2, 15.9744_MHz_XTAL / 4);
+	m_ctc2->intr_callback().set_inputline(m_maincpu, INPUT_LINE_IRQ0);
+	m_ctc2->zc_callback<1>().set(m_ctc2, FUNC(z80ctc_device::trg2));
+	// trg3 connected to /RQ signal
 
 	m_ram->set_default_size("256K");
 	RAM(config, m_videoram).set_default_size("4K").set_default_value(0x00);
